#include <xc.h>

#include "analog_read.h"

#include <stdlib.h>

#include "usb_device_cdc.h"


void AnalogInit(void) {
    TRISCbits.TRISC3 = 1;
    ANSELbits.ANS7 = 1; // Set PORT RC3 to analog input
    ADCON0 = 0b00011101; // ADC port channel 3 (AN3), Enable ADC
    ADCON1 = 0b00000000; // Use Internal Voltage Reference (Vdd and Vss)
    ADCON2 = 0b00111011; // Left justify result, 12 TAD, Select the FRC for 16 MHz
    
    //setup pwm
    TRISCbits.TRISC4 = 0;
    CCP1CON = 0b00001100;          //Single output: P1B, 
    PSTRCON = 0b00010010;          //Stg occurs on next PWM period & P1B pin has the PWM waveform 
    PWM1CON = 0x80;                //Auto start & Dead-band delay 
    //PR2 = 0xFF;                    //Period = 4 * TOSC * (PR2 + 1) * (TMR2 Prescale Value) MAX period(1.023e-3) 
    PR2 = 0xC7;                     //Period = 4 * TOSC * (PR2 + 1) * (TMR2 Prescale Value) (15Khz))
   // T2CON = 0x07;                  //No post scaler , 16 prescale & TMR2==ON     
    T2CON = 0x05;                    ///No post scaler , 4 prescale & TMR2==ON  
    
    //setup timer 1
    /*T1CKPS1 = 1;   // bits 5-4  Prescaler Rate Select bits
    T1CKPS0 = 1;   // bit 4
    T1OSCEN = 1;   // bit 3 Timer1 Oscillator Enable Control bit 1 = on
    T1SYNC = 1;    // bit 2 Timer1 External Clock Input Synchronization Control bit...1 = Do not synchronize external clock input
    TMR1CS = 0;    // bit 1 Timer1 Clock Source Select bit...0 = Internal clock (FOSC/4)
    TMR1ON = 1;    // bit 0 enables timer
    TMR1H = 0xC0;             // preset for timer1 MSB register
    TMR1L = 0;             // preset for timer1 LSB register
    TMR1IE  =   0;         // disable Timer1 interrupts*/
    TMR1IF = 0;        // clear timer1 interupt flag TMR1IF
    ADCON0bits.GO = 1;
}

void AnalogTask(void) {
    unsigned int ADCValue;
    unsigned int Dig_ConLow = 0; 
    /*if (TMR1IF) {
        TMR1H = 0xC0;
        TMR1IF =0;*/
        if (ADCON0bits.GO == 0) {
            //ADCValue = ((ADRESH << 8) | ADRESL) & 0x3FF;          //for right justify
            ADCValue = (((ADRESH << 8) | ADRESL) >> 6)  & 0x3FF ; //for left justify
            CCPR1L = ADRESH;            //MSB for 10-bit PWM duty cycle    ****DUTY CYCLE == A2D con********  
            Dig_ConLow = ADRESL;        //LSB of A2D con        ****DUTY CYCLE == A2D con******** 
            Dig_ConLow = Dig_ConLow >>2; // line up with DC1B<1:0>        ****DUTY CYCLE == A2D con******** 
            CCP1CON = CCP1CON | Dig_ConLow;
            ADCON0bits.GO = 1;
            
            
            /*if (USBUSARTIsTxTrfReady(FPGA_UART_CDC_PORT)) {
                printbuffer = utoa(printbuffer, ADCValue, 10);
                strcat(printbuffer, "\r");
                putrsUSBUSART(FPGA_UART_CDC_PORT, printbuffer);
                
            }*/
        }
    //}
}

/*
char *itoa(char *buffer, int i) {
    unsigned int n;
            unsigned int negate = 0;
            int c = 6;

    if (i < 0) {
        negate = 1;
                n = -i;
    } else if (i == 0) {
        buffer[0] = '0';
                buffer[1] = 0;
        return buffer;
    } else {
        n = i;
    }
    buffer[c--] = 0;
    do {
        buffer[c--] = (n % 10) + '0';
                n = n / 10;
    } while (n);
        if (negate) {
            buffer[c--] = '-';
        }
    return &buffer[c + 1];
}*/
